import { useSiteContext } from '@context/SiteContext'
import useLanguage from '@hooks/useLanguage'
import { useNavigate } from '@solidjs/router'
import { IconKey, IconUser } from '@tabler/icons-solidjs'
import { useFormHandler } from 'solid-form-handler'
import { yupSchema } from 'solid-form-handler/yup'
import { onMount } from 'solid-js'
import * as yup from 'yup'
import './login.scss'

import Logo from '@assets/logo.svg'
import TextInput from '@components/common/TextInput'
import useAuth from '@hooks/useAuth'
import useToast from '@hooks/useToast'

const loginSchema = yup.object({
  username: yup.string().required('Username is required'),
  password: yup.string().required('Password is required'),
})

const language = useLanguage()

export default function Login() {
  const { store, setAuth } = useSiteContext()
  const navigate = useNavigate()
  const { clickLogIn } = useAuth(setAuth)
  const notify = useToast()
  const formHandler = useFormHandler(yupSchema(loginSchema))
  const { formData } = formHandler

  onMount(() => {
    if (store.auth) {
      navigate('/', { replace: true })
    }
  })

  const submit = async (event) => {
    event.preventDefault()
    await formHandler.validateForm()
    try {
      const { username, password } = formData()
      await clickLogIn(username, password, formHandler.resetForm)
      notify.success({
        title: 'Login success!',
        description: 'Welcome back!',
        closable: true,
      })
    } catch (error) {
      notify.error({
        title: 'Login fail!',
        description: error?.data
          ? language.message[error.data]
          : 'Your username or password input is wrong!',
        closable: true,
      })
    }
  }

  return (
    <div class="login-page">
      <div class="card glass card-compact login-wrap shadow-xl">
        <div class="h-44">
          <picture class="logo">
            <source srcSet={Logo} type="image/png" media="(min-width: 600px)" />
            <img src={Logo} alt="logo" />
          </picture>
        </div>
        <div class="card-body">
          <h1 class="card-title">{language.ui.login}</h1>
          <form autoComplete="off" onSubmit={submit}>
            <TextInput
              name="username"
              placeholder="Username"
              icon={IconUser}
              formHandler={formHandler}
            />
            <TextInput
              name="password"
              type="password"
              placeholder="Password"
              icon={IconKey}
              formHandler={formHandler}
            />
            <div class="card-actions justify-end mt-5">
              <button type="submit" class="btn btn-primary">
                {language.ui.login}
              </button>
            </div>
          </form>
        </div>
      </div>
    </div>
  )
}
