import { putUpdateProfile } from '@api/user'
import TextInput from '@components/common/TextInput'
import { useSiteContext } from '@context/SiteContext'
import useLanguage from '@hooks/useLanguage'
import useToast from '@hooks/useToast'
import { IconKey, IconLetterN, IconUserCircle } from '@tabler/icons-solidjs'
import { Helpers } from '@utils/helper'
import { useFormHandler } from 'solid-form-handler'
import { yupSchema } from 'solid-form-handler/yup'
import { createEffect } from 'solid-js'
import * as yup from 'yup'

const profileSchema = yup.object({
  name: yup.string().required('Name is required'),
  password: yup.string().nullable().optional(),
  'confirm-password': yup.string().when('password', {
    is: (val) => !!(val && val.length > 0),
    then: (schema) =>
      schema.oneOf([yup.ref('password'), null], 'Passwords must match'),
  }),
})

const language = useLanguage()
const notify = useToast()

export default function Profile() {
  const {
    store: { userInfo },
    setUser,
  } = useSiteContext()
  const formHandler = useFormHandler(yupSchema(profileSchema))
  const { formData } = formHandler

  createEffect(() => {
    formHandler.fillForm({
      name: userInfo?.name,
    })
  })

  const submit = async (event) => {
    event.preventDefault()
    await formHandler.validateForm()
    try {
      const { name, password } = formData()
      const clearObj = Helpers.clearObject({
        name: name || null,
        password: password || null,
      })
      const resp = await putUpdateProfile(clearObj)

      if (resp.status === 200) {
        setUser(resp.data)
        formHandler.setFieldValue('password', '')
        formHandler.setFieldValue('confirm-password', '')
        notify.success({
          title: 'Update profile success!',
          description: 'Your profile has been updated!',
        })
      }
    } catch (error) {
      notify.error({
        title: 'Update profile fail!',
        description: error?.data
          ? language.message[error.data]
          : 'Your username or password input is wrong!',
        closable: true,
      })
    }
  }

  return (
    <div class="profile">
      <div class="divider divider-accent text-xl">
        <span class="text-green-400">
          <IconUserCircle size={30} />
        </span>
        {language.ui.profile}
      </div>
      <div class="card w-full bg-base-100 shadow-lg border border-gray-200">
        <div class="card-body">
          <form autoComplete="off" onSubmit={submit}>
            <p class="card-title">{language.ui.changeInfo}</p>
            <div class="form-content py-5">
              <TextInput
                icon={IconLetterN}
                name="name"
                placeholder={language.ui.displayName}
                formHandler={formHandler}
              />
              <TextInput
                icon={IconKey}
                name="password"
                type="password"
                placeholder={language.ui.newPassword}
                formHandler={formHandler}
              />
              <TextInput
                icon={IconKey}
                name="confirm-password"
                type="password"
                placeholder={language.ui.confirmNewPassword}
                formHandler={formHandler}
              />
            </div>
            <div class="card-actions">
              <button type="submit" class="btn btn-primary">
                {language.ui.save}
              </button>
            </div>
          </form>
        </div>
      </div>
    </div>
  )
}
